package com.stormtales.notevault.ui.home;

import android.app.Activity;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import android.widget.Toast;
import androidx.activity.result.ActivityResult;
import androidx.activity.result.ActivityResultCallback;
import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContracts;
import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;
import androidx.recyclerview.widget.DividerItemDecoration;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import com.google.android.material.floatingactionbutton.FloatingActionButton;
import com.stormtales.notevault.R;
import com.stormtales.notevault.data.entities.Song;
import com.stormtales.notevault.data.repositories.SongRepository;
import com.stormtales.notevault.databinding.FragmentHomeBinding;
import com.stormtales.notevault.ui.songeditor.SongEditorDialog;

import java.util.ArrayList;

public class HomeFragment extends Fragment {

    private FragmentHomeBinding binding;
    private RecyclerView recyclerView;
    private SongAdapter songAdapter;
    private HomeViewModel homeViewModel;
    private static final int PICK_FILE_REQUEST_CODE = 1;

    public View onCreateView(@NonNull LayoutInflater inflater,
                             ViewGroup container, Bundle savedInstanceState) {
        homeViewModel = new ViewModelProvider(this).get(HomeViewModel.class);
        homeViewModel.setSongRepository(new SongRepository(this.getContext()));

        binding = FragmentHomeBinding.inflate(inflater, container, false);
        View root = binding.getRoot();
        recyclerView = root.findViewById(R.id.recycler_view_songs);

        ActivityResultLauncher<Intent> launcher = registerForActivityResult(
                new ActivityResultContracts.StartActivityForResult(),
                activityResult -> {
                    getActivity();
                    if(activityResult.getResultCode() == Activity.RESULT_OK && activityResult.getData() != null) {
                        if(activityResult.getData().getClipData() != null) {
                            int fileNumber = activityResult.getData().getClipData().getItemCount();
                            Uri[] files = new Uri[fileNumber];
                            for(int i = 0; i < fileNumber; i++) {
                                files[i] = activityResult.getData().getClipData().getItemAt(i).getUri();
                                Toast.makeText(requireContext(), "Uri: " + files[i].toString(), Toast.LENGTH_SHORT).show();
                            }
                            handleSelectedNoteSheets(files);
                        } else {
                            Uri uri = activityResult.getData().getData();
                            Toast.makeText(requireContext(), "Uri: " + uri.toString(), Toast.LENGTH_SHORT).show();
                            handleSelectedNoteSheets(uri);
                        }
                    }
                }
        );

        FloatingActionButton importBtn = root.findViewById(R.id.addSongBtn);
        importBtn.setOnClickListener(v -> {
            Intent intent = new Intent(Intent.ACTION_OPEN_DOCUMENT);
            intent.setType("*/*"); // Alle Dateitypen

            String[] mimeTypes = {"application/pdf", "image/png", "image/jpeg"};
            intent.putExtra(Intent.EXTRA_MIME_TYPES, mimeTypes);
            intent.putExtra(Intent.EXTRA_ALLOW_MULTIPLE, true);

            intent.addCategory(Intent.CATEGORY_OPENABLE);


            launcher.launch(intent);
        });

        // RecyclerView einrichten
        songAdapter = new SongAdapter(new ArrayList<>(), new SongEventClickListener());
        recyclerView.setAdapter(songAdapter);
        recyclerView.setLayoutManager(new LinearLayoutManager(getContext()));
        DividerItemDecoration dividerItemDecoration = new DividerItemDecoration(recyclerView.getContext(), LinearLayoutManager.VERTICAL);
        recyclerView.addItemDecoration(dividerItemDecoration);

        // Beobachte Änderungen in der Song-Liste
        homeViewModel.getAllSongsLive().observe(getViewLifecycleOwner(), songs -> {
            songAdapter.updateData(songs);
        });

        return root;
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        binding = null;
    }

    private void handleSelectedNoteSheets(Uri... files) {
        SongEditorDialog songEditorDialog = new SongEditorDialog();
        songEditorDialog.setNoteSheetFiles(files);
        songEditorDialog.setHomeViewModel(homeViewModel);
        songEditorDialog.show(getParentFragmentManager(), "songEditorDialog");
    }

    private static class SongEventClickListener implements SongAdapter.OnSongEventClickListener {
        @Override
        public void onEditSong(Song song) {

        }

        @Override
        public void onDeleteSong(Song song) {

        }
    }
}